/*
 * Wii Classic Controller for Arduino 
 * 
 */

#include <Arduino.h>
#include <Wire.h>
#include "WiiClassicController.h"

#define CONTROLLER_ADDRESS 0x52

// Init I2C Interface
void WiiClassicController::init() {
  Wire.begin();

  sendByte(0x55, 0xF0);
  sendByte(0x00, 0xFB);

  scan();
}

// SCAN Wii Classic Controller Status
void WiiClassicController::scan() {
  // --- I2C Interface ---
  int count = 0;
  int readLen = Wire.requestFrom(CONTROLLER_ADDRESS, 8);
  while(Wire.available()) {
    values[count] = Wire.read();
    count++;
  }
  sendByte(0x00, 0x00);

  // --- DECODE RECIVE DATA ---
  if (readLen == 8 && count == 8) {
    // Analog data
    analogLX = values[0] & 0b00111111 ;
    analogLY = values[1] & 0b00111111 ;

    analogRX = ((values[0] & 0b11000000) >> 3) | (( values[1] & 0b11000000) >> 5) | (( values[2] & 0b10000000) >> 7);
    analogRY = values[2] & 0b00011111;
  
    analogRB=  values[3] & 0b00011111 ;
    analogLB =  ((values[2] & 0b01100000) >> 2) | ((values[3] & 0b11100000) >> 5)  ;
  
    // Digital data
    ((uint8_t *)&pushButton)[0] = ~values[4] ; // Reverse　Logic
    ((uint8_t *)&pushButton)[1] = ~values[5] ;
  }
}

// Wait to I2C Ready
void WiiClassicController::writeReadyCheck() {
  int errCount = 0 ;
  while(errCount < 100) {
    Wire.beginTransmission(CONTROLLER_ADDRESS);
    int ready = Wire.endTransmission(); // Check endTransmission is Nomal end.
    if (ready == 0) {
      break ;
    }
    errCount ++ ;
    delay(10) ;
    break;
  }
}

// SEND BYTE FOR Wii Classic Conrtroller
void WiiClassicController::sendByte(byte data, byte location)
{
  writeReadyCheck() ;

  Wire.beginTransmission(CONTROLLER_ADDRESS);
  Wire.write(location);
  Wire.write(data);
  Wire.endTransmission();

  delay(10);
}
