#include <Wire.h>
#include "WiiClassicController.h"
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#define BAUDRATE 9600
char str[128] ;

// ----- DEMO DISPLAY -----
#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 64 // OLED display height, in pixels
#define OLED_RESET     4 // Reset pin # (or -1 if sharing Arduino reset pin)
#define SCREEN_ADDRESS 0x3C ///< See datasheet for Address; 0x3D for 128x64, 0x3C for 128x32
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);

// ----- Wii Classic Controller -----
WiiClassicController wiiCC ;

uint8_t RX,RY,LX,LY,BL,BR ;
uint16_t BTN ;

void controlerDisp() {
  int ofLX = (wiiCC.analogLX -32)/ 2 ; // (0~64 -> -15~15)
  int ofLY = (wiiCC.analogLY -32)/ 2 ; 
  int ofRX = (wiiCC.analogRX - 11) ; // (0~32 -> -15 ~ 15)
  int ofRY = (wiiCC.analogRY - 11) ;
  int ofBL = wiiCC.analogLB * (32./54) + 1 ; // (0~32 -> 1~55)
  int ofBR = wiiCC.analogRB * (32./54) + 1 ;

  display.clearDisplay();

  display.drawCircle( 16 , 46 , 15 , SSD1306_WHITE); // 左スティック
  display.drawCircle(111 , 46 , 15 , SSD1306_WHITE); // 右スティック
  display.fillCircle( 16 + ofLX , 46 - ofLY , 3, SSD1306_INVERSE); // 左スティック位置
  display.fillCircle(111 + ofRX , 46 - ofRY , 3, SSD1306_INVERSE); // 右スティック位置

  display.drawRect( 34, 55 , 28 , 6 , SSD1306_WHITE); // アナログ左
  display.drawRect( 65, 55 , 28 , 6 , SSD1306_WHITE); // アナログ右
  display.fillRect( 34, 55 , ofBL , 6 , SSD1306_WHITE); // アナログ左レベル
  display.fillRect( 93-ofBR, 55 , ofBR , 6 , SSD1306_WHITE); // アナログ右レベル

  if (wiiCC.isLBtn()) {
    display.fillRect( 34, 45 , 10 , 6 , SSD1306_WHITE); // アナログ左 Digital
  } else {
    display.drawRect( 34, 45 , 10 , 6 , SSD1306_WHITE); // アナログ左 Digital
  }  
  if (wiiCC.isRBtn()) {
    display.fillRect( 83, 45 , 10 , 6 , SSD1306_WHITE); // アナログ右 Digital
  } else {
    display.drawRect( 83, 45 , 10 , 6 , SSD1306_WHITE); // アナログ右 Digital
  }  
  if (wiiCC.isZLBtn()) {
    display.fillRect( 46, 45 , 10 , 6 , SSD1306_WHITE); // ZL
  } else {
    display.drawRect( 46, 45 , 10 , 6 , SSD1306_WHITE); // ZL
  }
  if (wiiCC.isZRBtn()) {
    display.fillRect( 71, 45 , 10 , 6 , SSD1306_WHITE); // ZR
  } else {
    display.drawRect( 71, 45 , 10 , 6 , SSD1306_WHITE); // ZR
  }

  if (wiiCC.isLeft()) {
    display.fillRect( 1, 9 , 12 , 12 , SSD1306_WHITE); // ←
  } else {
   display.drawRect( 1, 9 , 12 , 12 , SSD1306_WHITE); // ←
  }
  if (wiiCC.isRight()) {
   display.fillRect(29, 9 , 12 , 12 , SSD1306_WHITE); // →
  } else {
    display.drawRect(29, 9 , 12 , 12 , SSD1306_WHITE); // →
  }
  if (wiiCC.isUp()) {
    display.fillRect( 15, 2 , 12 , 12 , SSD1306_WHITE); // ↑
  } else {
   display.drawRect( 15, 2 , 12 , 12 , SSD1306_WHITE); // ↑
  }
  if (wiiCC.isDown()) {
    display.fillRect( 15,16 , 12 , 12 , SSD1306_WHITE); // ↓
  } else {
    display.drawRect( 15,16 , 12 , 12 , SSD1306_WHITE); // ↓
  }

  if (wiiCC.isSelectBtn()) {
    display.fillCircle( 51 , 18 , 3 , SSD1306_WHITE); // -
  } else {
    display.drawCircle( 51 , 18 , 3 , SSD1306_WHITE); // -
  }
  if (wiiCC.isHomeBtn()) {
    display.fillCircle( 59 , 18 , 3 , SSD1306_WHITE); // HOME
  } else {
    display.drawCircle( 59 , 18 , 3 , SSD1306_WHITE); // HOME
  }
  if (wiiCC.isStartBtn()) {
    display.fillCircle( 67 , 18 , 3 , SSD1306_WHITE); // +
  } else {
    display.drawCircle( 67 , 18 , 3 , SSD1306_WHITE); // +
  }

  if (wiiCC.isXBtn()) {
    display.fillCircle( 103 , 7 , 6 , SSD1306_WHITE); // x
  } else {
    display.drawCircle( 103 , 7 , 6 , SSD1306_WHITE); // x
  }
  if (wiiCC.isYBtn()) {
    display.fillCircle( 87 , 13 , 6 , SSD1306_WHITE); // y
  } else {
    display.drawCircle( 87 , 13 , 6 , SSD1306_WHITE); // y
  }
  if (wiiCC.isABtn()) {
    display.fillCircle( 109 ,21 , 6 , SSD1306_WHITE); // a
  } else {
    display.drawCircle( 109 ,21 , 6 , SSD1306_WHITE); // a
  }
  if (wiiCC.isBBtn()) {
    display.fillCircle(  93, 27 , 6 , SSD1306_WHITE); // b
  } else {
    display.drawCircle(  93, 27 , 6 , SSD1306_WHITE); // b
  }
  display.display(); // Update screen with each newly-drawn rectangle
  delay(10) ; // RaspberryPi Pico は速すぎるのでココにディレイを入れないとSSD1306がハングアップする。
              // Adafruteのライブラリが更新されてRadyを見るようになれば解消されるかも。
              // Arduinoの場合はこのディレイは不要
}

void setup() {
  Serial.begin(BAUDRATE);
  Wire.begin();

  wiiCC.init( ) ;

  // SSD1306_SWITCHCAPVCC = generate display voltage from 3.3V internally
  if(!display.begin(SSD1306_SWITCHCAPVCC, SCREEN_ADDRESS)) {
    Serial.println(F("SSD1306 allocation failed"));
    for(;;); // Don't proceed, loop forever
  }
  // Clear the buffer
  display.clearDisplay();
}

void loop() {
  wiiCC.scan( ) ;
  if (LX != wiiCC.analogLX || LY != wiiCC.analogLY || 
      RX != wiiCC.analogRY || RY != wiiCC.analogRY || 
      BL != wiiCC.analogLB || BR != wiiCC.analogLB || BTN != wiiCC.pushButton) {
    LX = wiiCC.analogLX ;
    LY = wiiCC.analogLY ; 
    RX = wiiCC.analogRX ;
    RY = wiiCC.analogRY ;
    BL = wiiCC.analogLB ;
    BR = wiiCC.analogRB ;
    BTN = wiiCC.pushButton ;

    sprintf(str,"L:%3d,%3d R:%3d,%3d B:%3d,%3d B:%04X ",LX,LY,RX,RY,BL,BR,BTN) ;
    Serial.print(str);

    if (wiiCC.isLeft()) { Serial.print("← "); }
    if (wiiCC.isRight()){ Serial.print("→ "); }
    if (wiiCC.isUp())   { Serial.print("↑ "); }
    if (wiiCC.isDown()) { Serial.print("↓ "); }
    if (wiiCC.isLBtn()) { Serial.print("LB "); }
    if (wiiCC.isRBtn()) { Serial.print("RB "); }
    if (wiiCC.isZLBtn()) { Serial.print("ZL "); }
    if (wiiCC.isZRBtn()) { Serial.print("ZR "); }
    if (wiiCC.isSelectBtn()) { Serial.print("- "); }
    if (wiiCC.isHomeBtn()) { Serial.print("HOME "); }
    if (wiiCC.isStartBtn()) { Serial.print("+ "); }
    if (wiiCC.isXBtn()) { Serial.print("X "); }
    if (wiiCC.isYBtn()) { Serial.print("Y "); }
    if (wiiCC.isABtn()) { Serial.print("a "); }
    if (wiiCC.isBBtn()) { Serial.print("b "); }
    Serial.println() ;

    controlerDisp();
  }
}
